@@ -109,6 +109,10 @@ processes
 
     bokeh serve app_script.py --num-procs 2
 
+Note that due to limitations inherent in Tornado, Windows does not support
+``--num-procs`` values greater than one! In this case consider running multiple
+Bokeh server instances behind a load balancer.
+
 By default, cross site connections to the Bokeh server websocket are not
 allowed. You can enable websocket connections originating from additional
 hosts by specifying them with the ``--allow-websocket-origin`` option:
@@ -299,7 +303,7 @@ from os import getpid
 from ..subcommand import Subcommand
 from ..util import build_single_handler_applications, die, report_server_init_errors
 
-LOGLEVELS = ('debug', 'info', 'warning', 'error', 'critical')
+LOGLEVELS = ('trace', 'debug', 'info', 'warning', 'error', 'critical')
 SESSION_ID_MODES = ('unsigned', 'signed', 'external-signed')
 DEFAULT_LOG_FORMAT = "%(asctime)s %(message)s"
 
