@@ -1,21 +1,37 @@
 from __future__ import absolute_import
 
 import datetime
+import time
 import unittest
 import numpy as np
 import pandas as pd
 from copy import copy
 
+import pytest
+
 from bokeh.core.properties import (field, value,
-    NumberSpec, ColorSpec, Bool, Int, Float, Complex, String,
-    Regex, Seq, List, Dict, Tuple, Array, Instance, Any, Interval, Either,
-    Enum, Color, DashPattern, Size, Percent, Angle, AngleSpec,
-    DistanceSpec, FontSizeSpec, Override, Include, MinMaxBounds, TitleProp)
+    NumberSpec, ColorSpec, Bool, Int, Float, Complex, Date, String,
+    Regex, Seq, List, Dict, Tuple, Instance, Any, Interval, Either,
+    Enum, Color, DashPattern, Size, Percent, Angle, AngleSpec, StringSpec,
+    DistanceSpec, FontSize, FontSizeSpec, Override, Include, MinMaxBounds,
+    DataDistanceSpec, ScreenDistanceSpec, ColumnData, UnitsSpec)
+
+from bokeh.core.property.containers import PropertyValueColumnData, PropertyValueDict, PropertyValueList
 
 from bokeh.core.has_props import HasProps
 
 from bokeh.models import Plot
-from bokeh.models.annotations import Title
+
+class Test_Date(object):
+    def test_validate_seconds(self):
+        t = time.time()
+        d = Date()
+        assert d.transform(t) == datetime.date.today()
+
+    def test_validate_milliseconds(self):
+        t = time.time() * 1000
+        d = Date()
+        assert d.transform(t) == datetime.date.today()
 
 class Basictest(unittest.TestCase):
 
@@ -23,7 +39,8 @@ class Basictest(unittest.TestCase):
         class Foo(HasProps):
             x = Int(12)
             y = String("hello")
-            z = Array(Int, np.array([1, 2, 3]))
+            z = List(Int, [1, 2, 3])
+            zz = Dict(String, Int)
             s = String(None)
 
         f = Foo()
@@ -33,14 +50,10 @@ class Basictest(unittest.TestCase):
         self.assertEqual(f.s, None)
 
 
-        self.assertEqual(set(["x", "y", "z", "s"]), f.properties())
+        self.assertEqual(set(["x", "y", "z", "zz", "s"]), f.properties())
         with_defaults = f.properties_with_values(include_defaults=True)
-        del with_defaults['z'] # can't compare equality on the np array
-        self.assertDictEqual(dict(x=12, y="hello", s=None), with_defaults)
+        self.assertDictEqual(dict(x=12, y="hello", z=[1,2,3], zz={}, s=None), with_defaults)
         without_defaults = f.properties_with_values(include_defaults=False)
-        # the Array is in here because it's mutable
-        self.assertTrue('z' in without_defaults)
-        del without_defaults['z']
         self.assertDictEqual(dict(), without_defaults)
 
         f.x = 18
@@ -50,9 +63,18 @@ class Basictest(unittest.TestCase):
         self.assertEqual(f.y, "bar")
 
         without_defaults = f.properties_with_values(include_defaults=False)
-        del without_defaults['z']
         self.assertDictEqual(dict(x=18, y="bar"), without_defaults)
 
+        f.z[0] = 100
+
+        without_defaults = f.properties_with_values(include_defaults=False)
+        self.assertDictEqual(dict(x=18, y="bar", z=[100,2,3]), without_defaults)
+
+        f.zz = {'a': 10}
+
+        without_defaults = f.properties_with_values(include_defaults=False)
+        self.assertDictEqual(dict(x=18, y="bar", z=[100,2,3], zz={'a': 10}), without_defaults)
+
     def test_enum(self):
         class Foo(HasProps):
             x = Enum("blue", "red", "green")     # the first item is the default
@@ -445,6 +467,61 @@ class TestNumberSpec(unittest.TestCase):
         f.x = None
         self.assertIs(Foo.__dict__["x"].serializable_value(f), None)
 
+    def tests_accepts_timedelta(self):
+        class Foo(HasProps):
+            dt = NumberSpec("dt", accept_datetime=True)
+            ndt = NumberSpec("ndt", accept_datetime=False)
+
+        f = Foo()
+
+        f.dt = datetime.timedelta(3, 54)
+        self.assertEqual(f.dt, 259254000.0)
+
+        # counts as number.Real out of the box
+        f.dt = np.timedelta64(3000, "ms")
+        self.assertEqual(f.dt, np.timedelta64(3000, "ms"))
+
+        # counts as number.Real out of the box
+        f.dt = pd.Timedelta("3000ms")
+        self.assertEqual(f.dt, 3000.0)
+
+
+        f.ndt = datetime.timedelta(3, 54)
+        self.assertEqual(f.ndt, 259254000.0)
+
+        # counts as number.Real out of the box
+        f.ndt = np.timedelta64(3000, "ms")
+        self.assertEqual(f.ndt, np.timedelta64(3000, "ms"))
+
+        f.ndt = pd.Timedelta("3000ms")
+        self.assertEqual(f.ndt, 3000.0)
+
+    def test_accepts_datetime(self):
+        class Foo(HasProps):
+            dt = NumberSpec("dt", accept_datetime=True)
+            ndt = NumberSpec("ndt", accept_datetime=False)
+
+        f = Foo()
+
+        f.dt = datetime.datetime(2016, 5, 11)
+        self.assertEqual(f.dt, 1462924800000.0)
+
+        f.dt = datetime.date(2016, 5, 11)
+        self.assertEqual(f.dt, 1462924800000.0)
+
+        f.dt = np.datetime64("2016-05-11")
+        self.assertEqual(f.dt, 1462924800000.0)
+
+
+        with self.assertRaises(ValueError):
+            f.ndt = datetime.datetime(2016, 5, 11)
+
+        with self.assertRaises(ValueError):
+            f.ndt = datetime.date(2016, 5, 11)
+
+        with self.assertRaises(ValueError):
+            f.ndt = np.datetime64("2016-05-11")
+
     def test_default(self):
         class Foo(HasProps):
             y = NumberSpec(default=12)
@@ -526,12 +603,12 @@ class TestFontSizeSpec(unittest.TestCase):
 
             v = '10%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             v = '10.2%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             f = '_10%s' % unit
@@ -547,12 +624,12 @@ class TestFontSizeSpec(unittest.TestCase):
         for unit in css_units.upper().split("|"):
             v = '10%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             v = '10.2%s' % unit
             a.x = v
-            self.assertEqual(a.x, dict(value=v))
+            self.assertEqual(a.x, v)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(value=v))
 
             f = '_10%s' % unit
@@ -565,6 +642,39 @@ class TestFontSizeSpec(unittest.TestCase):
             self.assertEqual(a.x, f)
             self.assertEqual(a.lookup('x').serializable_value(a), dict(field=f))
 
+    def test_bad_font_size_values(self):
+        class Foo(HasProps):
+            x = FontSizeSpec(default=None)
+
+        a = Foo()
+
+        with self.assertRaises(ValueError):
+            a.x = "6"
+
+        with self.assertRaises(ValueError):
+            a.x = 6
+
+        with self.assertRaises(ValueError):
+            a.x = ""
+
+    def test_fields(self):
+        class Foo(HasProps):
+            x = FontSizeSpec(default=None)
+
+        a = Foo()
+
+        a.x = "_120"
+        self.assertEqual(a.x, "_120")
+
+        a.x = dict(field="_120")
+        self.assertEqual(a.x, dict(field="_120"))
+
+        a.x = "foo"
+        self.assertEqual(a.x, "foo")
+
+        a.x = dict(field="foo")
+        self.assertEqual(a.x, dict(field="foo"))
+
 class TestAngleSpec(unittest.TestCase):
     def test_default_none(self):
         class Foo(HasProps):
@@ -1094,6 +1204,7 @@ class TestProperties(unittest.TestCase):
         prop = String()
 
         self.assertTrue(prop.is_valid(None))
+
         self.assertFalse(prop.is_valid(False))
         self.assertFalse(prop.is_valid(True))
         self.assertFalse(prop.is_valid(0))
@@ -1102,11 +1213,60 @@ class TestProperties(unittest.TestCase):
         self.assertFalse(prop.is_valid(1.0))
         self.assertFalse(prop.is_valid(1.0+1.0j))
         self.assertTrue(prop.is_valid(""))
+        self.assertTrue(prop.is_valid("6"))
         self.assertFalse(prop.is_valid(()))
         self.assertFalse(prop.is_valid([]))
         self.assertFalse(prop.is_valid({}))
         self.assertFalse(prop.is_valid(Foo()))
 
+    def test_FontSize(self):
+
+        prop = FontSize()
+
+        self.assertTrue(prop.is_valid(None))
+
+        css_units = "%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px"
+
+        for unit in css_units.split("|"):
+            v = '10%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '10.2%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '_10%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+            v = '_10.2%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+        for unit in css_units.upper().split("|"):
+            v = '10%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '10.2%s' % unit
+            self.assertTrue(prop.is_valid(v))
+
+            v = '_10%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+            v = '_10.2%s' % unit
+            self.assertFalse(prop.is_valid(v))
+
+        self.assertFalse(prop.is_valid(False))
+        self.assertFalse(prop.is_valid(True))
+        self.assertFalse(prop.is_valid(0))
+        self.assertFalse(prop.is_valid(1))
+        self.assertFalse(prop.is_valid(0.0))
+        self.assertFalse(prop.is_valid(1.0))
+        self.assertFalse(prop.is_valid(1.0+1.0j))
+        self.assertFalse(prop.is_valid(""))
+        self.assertFalse(prop.is_valid(()))
+        self.assertFalse(prop.is_valid([]))
+        self.assertFalse(prop.is_valid({}))
+        self.assertFalse(prop.is_valid(Foo()))
+
+
     def test_Regex(self):
         with self.assertRaises(TypeError):
             prop = Regex()
@@ -1652,19 +1812,106 @@ bokeh.core.tests.test_properties.Foo5(
     foo6=bokeh.core.tests.test_properties.Foo6(
         foo5=bokeh.core.tests.test_properties.Foo5(...)))"""
 
-def test_titleprop_transforms_string_into_title_object():
-    class Foo(HasProps):
-        title = TitleProp
-    f = Foo(title="hello")
-    assert isinstance(f.title, Title)
-    assert f.title.text == "hello"
-
 def test_field_function():
     assert field("foo") == dict(field="foo")
-    # TODO (bev) would like this to work I think
-    #assert field("foo", transform="junk") == dict(field="foo", transform="junk")
+    assert field("foo", "junk") == dict(field="foo", transform="junk")
+    assert field("foo", transform="junk") == dict(field="foo", transform="junk")
 
 def test_value_function():
     assert value("foo") == dict(value="foo")
-    # TODO (bev) would like this to work I think
-    #assert value("foo", transform="junk") == dict(value="foo", transform="junk")
+    assert value("foo", "junk") == dict(value="foo", transform="junk")
+    assert value("foo", transform="junk") == dict(value="foo", transform="junk")
+
+def test_strict_dataspec_key_values():
+    for typ in (NumberSpec, StringSpec, FontSizeSpec, ColorSpec, DataDistanceSpec, ScreenDistanceSpec):
+        class Foo(HasProps):
+            x = typ("x")
+        f = Foo()
+        with pytest.raises(ValueError):
+            f.x = dict(field="foo", units="junk")
+
+def test_dataspec_dict_to_serializable():
+    for typ in (NumberSpec, StringSpec, FontSizeSpec, ColorSpec):
+        class Foo(HasProps):
+            x = typ("x")
+        foo = Foo(x=dict(field='foo'))
+        props = foo.properties_with_values(include_defaults=False)
+        assert props['x']['field'] == 'foo'
+        assert props['x'] is not foo.x
+
+def test_DataDistanceSpec():
+    assert issubclass(DataDistanceSpec, UnitsSpec)
+    class Foo(HasProps):
+        x = DataDistanceSpec("x")
+    foo = Foo(x=dict(field='foo'))
+    props = foo.properties_with_values(include_defaults=False)
+    assert props['x']['units'] == 'data'
+    assert props['x']['field'] == 'foo'
+    assert props['x'] is not foo.x
+
+def test_ScreenDistanceSpec():
+    assert issubclass(ScreenDistanceSpec, UnitsSpec)
+    class Foo(HasProps):
+        x = ScreenDistanceSpec("x")
+    foo = Foo(x=dict(field='foo'))
+    props = foo.properties_with_values(include_defaults=False)
+    assert props['x']['units'] == 'screen'
+    assert props['x']['field'] == 'foo'
+    assert props['x'] is not foo.x
+
+
+def test_strict_unitspec_key_values():
+    class FooUnits(HasProps):
+        x = DistanceSpec("x")
+    f = FooUnits()
+    f.x = dict(field="foo", units="screen")
+    with pytest.raises(ValueError):
+        f.x = dict(field="foo", units="junk", foo="crap")
+    class FooUnits(HasProps):
+        x = AngleSpec("x")
+    f = FooUnits()
+    f.x = dict(field="foo", units="deg")
+    with pytest.raises(ValueError):
+        f.x = dict(field="foo", units="junk", foo="crap")
+
+def test_Property_wrap():
+    for x in (Bool, Int, Float, Complex, String, Enum, Color,
+              Regex, Seq, Tuple, Instance, Any, Interval, Either,
+              DashPattern, Size, Percent, Angle, MinMaxBounds):
+        for y in (0, 1, 2.3, "foo", None, (), [], {}):
+            r = x.wrap(y)
+            assert r == y
+            assert isinstance(r, type(y))
+
+def test_List_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), {}):
+        r = List.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = List.wrap([1,2,3])
+    assert r == [1,2,3]
+    assert isinstance(r, PropertyValueList)
+    r2 = List.wrap(r)
+    assert r is r2
+
+def test_Dict_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), []):
+        r = Dict.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = Dict.wrap(dict(a=1, b=2))
+    assert r == dict(a=1, b=2)
+    assert isinstance(r, PropertyValueDict)
+    r2 = Dict.wrap(r)
+    assert r is r2
+
+def test_ColumnData_wrap():
+    for y in (0, 1, 2.3, "foo", None, (), []):
+        r = ColumnData.wrap(y)
+        assert r == y
+        assert isinstance(r, type(y))
+    r = ColumnData.wrap(dict(a=1, b=2))
+    assert r == dict(a=1, b=2)
+    assert isinstance(r, PropertyValueColumnData)
+    r2 = ColumnData.wrap(r)
+    assert r is r2
