@@ -22,19 +22,27 @@ always be active regardless of what other tools are currently active.
 '''
 from __future__ import absolute_import
 
-from ..core.enums import Anchor, Dimension, Dimensions, Location, TooltipFieldFormatter
+from ..core.enums import (Anchor, Dimension, Dimensions, Location,
+                          TooltipFieldFormatter, TooltipAttachment)
 from ..core.has_props import abstract
 from ..core.properties import (
-    Auto, Bool, Color, Dict, Either, Enum, Float, Percent, Instance, List,
-    Override, Seq, String, Tuple
+    Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Int, Float,
+    Percent, Instance, List, Seq, String, Tuple
+)
+from ..core.validation import error
+from ..core.validation.errors import (
+    INCOMPATIBLE_BOX_EDIT_RENDERER, INCOMPATIBLE_POINT_DRAW_RENDERER,
+    INCOMPATIBLE_POLY_DRAW_RENDERER, INCOMPATIBLE_POLY_EDIT_RENDERER,
+    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER
 )
 from ..model import Model
 from ..util.deprecation import deprecated
 
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
-from .renderers import Renderer
-from .layouts import Box, LayoutDOM
+from .glyphs import XYGlyph, Rect, Patches, MultiLine
+from .renderers import Renderer, GlyphRenderer
+from .layouts import LayoutDOM
 
 
 @abstract
@@ -43,8 +51,6 @@ class Tool(Model):
 
     '''
 
-    __deprecated_attributes__ = ["plot"]
-
     @property
     def plot(self):
         deprecated("Tool.plot property is no longer needed, and any use deprecated. In the future, accessing Tool.plot will result in an AttributeError")
@@ -96,7 +102,7 @@ class Inspection(Tool):
     """)
 
 @abstract
-class ToolbarBase(LayoutDOM):
+class ToolbarBase(Model):
     ''' A base class for different toolbars.
 
     '''
@@ -110,11 +116,6 @@ class ToolbarBase(LayoutDOM):
     A list of tools to add to the plot.
     """)
 
-    # This is an odd case. The sizing is custom handled. In the future we will
-    # probably set it as `stretch_width` or `stretch_height` depending on its
-    # orientation.
-    sizing_mode = Override(default=None)
-
 
 class Toolbar(ToolbarBase):
     ''' Collect tools to display for a single plot.
@@ -138,35 +139,20 @@ class Toolbar(ToolbarBase):
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
+class ProxyToolbar(ToolbarBase):
+    ''' A toolbar that allow to merge and proxy tools of toolbars in multiple plots. '''
 
-class ToolbarBox(Box):
+class ToolbarBox(LayoutDOM):
     ''' A layoutable toolbar that can accept the tools of multiple plots, and
     can merge the tools into a single button for convenience.
 
     '''
-    def _check_empty_layout(self):
-        # Overriding the children check from Box. As toolbarbox's children
-        # are normally set JS side.
-        return None
 
-    toolbar_location = Enum(Location, default='right', help="""
-        Should the toolbar be presented as if it was stuck to the `above`, `right`, `left`, `below`
-        edge of a plot. Default is `right`.
-    """)
-
-    tools = List(Instance(Tool), help="""
-    A list of tools to add to the plot.
-    """)
-
-    merge_tools = Bool(default=True, help="""
-        Merge all the tools together so there is one tool to control all the plots.
-    """)
-
-    logo = Enum("normal", "grey", help="""
-    What version of the Bokeh logo to display on the toolbar. If
-    set to None, no logo will be displayed.
+    toolbar = Instance(ToolbarBase, help="""
+    A toolbar associated with a plot which holds all its tools.
     """)
 
+    toolbar_location = Enum(Location, default="right")
 
 class PanTool(Drag):
     ''' *toolbar icon*: |pan_icon|
@@ -264,16 +250,7 @@ class ResetTool(Action):
 
     '''
 
-    reset_size = Bool(default=True, help="""
-    Whether activating the Reset tool should also reset the plot's canvas
-    dimensions to their original size.
-    """)
-
-
-class ResizeTool(object):
-    ''' DEPRECATED in 0.12.7, attempting to use will be a NO-OP
-
-    '''
+    pass
 
 
 class TapTool(Tap):
@@ -679,8 +656,13 @@ class HoverTool(Inspection):
     off by setting ``tooltips=None``.
 
     .. warning::
+        When supplying a callback or custom template, the explicit intent
+        of this Bokeh Model is to embed *raw HTML and  JavaScript code* for
+        a browser to execute. If any part of the code is derived from untrusted
+        user inputs, then you must take appropriate care to sanitize the user
+        input prior to passing to Bokeh.
 
-        Hover tool does not currently work with the following glyphs:
+    Hover tool does not currently work with the following glyphs:
 
         .. hlist::
             :columns: 3
@@ -725,7 +707,7 @@ class HoverTool(Inspection):
             default=[
                 ("index","$index"),
                 ("data (x, y)","($x, $y)"),
-                ("canvas (x, y)","($sx, $sy)"),
+                ("screen (x, y)","($sx, $sy)"),
             ], help="""
     The (name, field) pairs describing what the hover tool should
     display when there is a hit.
@@ -842,8 +824,10 @@ class HoverTool(Inspection):
     point of a tooltip. The default is to attach to the center of a glyph.
     """)
 
-    attachment = Enum("horizontal", "vertical", help="""
-    Whether tooltip's arrow should appear in the horizontal or vertical dimension.
+    attachment = Enum(TooltipAttachment, help="""
+    Whether the tooltip should be displayed to the left or right of the cursor
+    position or above or below it, or if it should be automatically placed
+    in the horizontal or vertical dimension.
     """)
 
     show_arrow = Bool(default=True, help="""
@@ -888,3 +872,227 @@ class RedoTool(Action):
         :height: 18pt
 
     '''
+
+@abstract
+class EditTool(Tool):
+    ''' A base class for all interactive draw tool types.
+
+    '''
+
+    empty_value = Either(Bool, Int, Float, Date, Datetime, Color, help="""
+    Defines the value to insert on non-coordinate columns when a new
+    glyph is inserted into the ColumnDataSource columns, e.g. when a
+    circle glyph defines 'x', 'y' and 'color' columns, adding a new
+    point will add the x and y-coordinates to 'x' and 'y' columns and
+    the color column will be filled with the defined empty value.
+    """)
+
+    renderers = List(Instance(Renderer), help="""
+    An explicit list of renderers corresponding to scatter glyphs
+    that may be edited.
+    """)
+
+class BoxEditTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |box_edit_icon|
+
+    The BoxEditTool allows drawing, dragging and deleting ``Rect``
+    glyphs on one or more renderers by editing the underlying
+    ``ColumnDataSource`` data. Like other drawing tools, the renderers
+    that are to be edited must be supplied explicitly as a list. When
+    drawing a new box the data will always be added to the
+    ``ColumnDataSource`` on the first supplied renderer.
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``x``, ``y``, ``width`` and ``height`` values
+    of the glyph. Any additional columns in the data source will be
+    padded with the declared ``empty_value``, when adding a new box.
+
+    The supported actions include:
+
+    * Add box: Hold shift then click and drag anywhere on the plot.
+
+    * Move box: Click and drag an existing box, the box will be
+      dropped once you let go of the mouse button.
+
+    * Delete box: Tap a box to select it then press <<backspace>> key
+      while the mouse is within the plot area.
+
+    To **Move** or **Delete** multiple boxes at once:
+
+    * Move selection: Select box(es) with <<shift>>+tap (or another
+      selection tool) then drag anywhere on the plot. Selecting and
+      then dragging on a specific box will move both.
+
+    * Delete selection: Select box(es) with <<shift>>+tap (or another
+      selection tool) then press <<backspace>> while the mouse is
+      within the plot area.
+
+    .. |box_edit_icon| image:: /_images/icons/BoxEdit.png
+        :height: 18pt
+    '''
+
+    dimensions = Enum(Dimensions, default="both", help="""
+    Which dimensions the box drawing is to be free in. By default,
+    users may freely draw boxes with any dimensions. If only "width"
+    is supplied, the box will be constrained to span the entire
+    vertical space of the plot, only the horizontal dimension can be
+    controlled. If only "height" is supplied, the box will be
+    constrained to span the entire horizontal space of the plot, and
+    the vertical dimension can be controlled.
+    """)
+
+    @error(INCOMPATIBLE_BOX_EDIT_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, Rect):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            return "%s glyph type(s) found." % glyph_types
+
+class PointDrawTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |point_draw_icon|
+
+    The PointDrawTool allows adding, dragging and deleting point-like
+    glyphs (of ``XYGlyph`` type) on one or more renderers by editing
+    the underlying ``ColumnDataSource`` data. Like other drawing
+    tools, the renderers that are to be edited must be supplied
+    explicitly as a list. Any newly added points will be inserted on
+    the ``ColumnDataSource`` of the first supplied renderer.
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``x`` and ``y`` values of the glyph. Any
+    additional columns in the data source will be padded with the
+    declared ``empty_value``, when adding a new point.
+
+    The supported actions include:
+
+    * Add point: Tap anywhere on the plot
+
+    * Move point: Tap and drag an existing point, the point will be
+      dropped once you let go of the mouse button.
+
+    * Delete point: Tap a point to select it then press <<backspace>>
+      key while the mouse is within the plot area.
+
+    .. |point_draw_icon| image:: /_images/icons/PointDraw.png
+        :height: 18pt
+    '''
+
+    add = Bool(default=True, help="""
+    Enables adding of new points on tap events.""")
+
+    drag = Bool(default=True, help="""
+    Enables dragging of existing points on pan events.""")
+
+    @error(INCOMPATIBLE_POINT_DRAW_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, XYGlyph):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            return "%s glyph type(s) found." % glyph_types
+
+class PolyDrawTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |poly_draw_icon|
+
+    The PolyDrawTool allows drawing, selecting and deleting
+    ``Patches`` and ``MultiLine`` glyphs on one or more renderers by
+    editing the underlying ColumnDataSource data. Like other drawing
+    tools, the renderers that are to be edited must be supplied
+    explicitly as a list.
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``xs`` and ``ys`` values of the glyph. Any
+    additional columns in the data source will be padded with the
+    declared ``empty_value``, when adding a new point.
+
+    The supported actions include:
+
+    * Add patch/multi-line: Double tap to add the first vertex, then
+      use tap to add each subsequent vertex, to finalize the draw
+      action double tap to insert the final vertex or press the <<esc>
+      key.
+
+    * Move patch/multi-line: Tap and drag an existing
+      patch/multi-line, the point will be dropped once you let go of
+      the mouse button.
+
+    * Delete patch/multi-line: Tap a patch/multi-line to select it
+      then press <<backspace>> key while the mouse is within the plot
+      area.
+
+    .. |poly_draw_icon| image:: /_images/icons/PolyDraw.png
+        :height: 18pt
+    '''
+
+    drag = Bool(default=True, help="""
+    Enables dragging of existing patches and multi-lines on pan events.""")
+
+    @error(INCOMPATIBLE_POLY_DRAW_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, (MultiLine, Patches)):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            return "%s glyph type(s) found." % glyph_types
+
+
+class PolyEditTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |poly_edit_icon|
+
+    The PolyEditTool allows editing the vertices of one or more
+    ``Patches`` or ``MultiLine`` glyphs. The glyphs to be edited can
+    be defined via the ``renderers`` property and the renderer for the
+    vertices can be defined via the ``vertex_renderer``, which must
+    render a point-like Glyph (of ``XYGlyph`` type).
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``xs`` and ``ys`` values of the glyph. Any
+    additional columns in the data source will be padded with the
+    declared ``empty_value``, when adding a new point.
+
+    The supported actions include:
+
+    * Show vertices: Double tap an existing patch or multi-line
+
+    * Add vertex: Double tap an existing vertex to select it, the tool
+      will draw the next point, to add it tap in a new location. To
+      finish editing and add a point double tap otherwise press the
+      <<esc> key to cancel.
+
+    * Move vertex: Drag an existing vertex and let go of the mouse
+      button to release it.
+
+    * Delete vertex: After selecting one or more vertices press
+      <<backspace>> while the mouse cursor is within the plot area.
+
+    .. |poly_edit_icon| image:: /_images/icons/PolyEdit.png
+        :height: 18pt
+    '''
+
+    vertex_renderer = Instance(GlyphRenderer, help="""
+    The renderer used to render the vertices of a selected line or
+    polygon.""")
+
+    @error(INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER)
+    def _check_compatible_vertex_renderer(self):
+        glyph = self.vertex_renderer.glyph
+        if not isinstance(glyph, XYGlyph):
+            return "glyph type %s found." % type(glyph).__name__
+
+    @error(INCOMPATIBLE_POLY_EDIT_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, (MultiLine, Patches)):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join([type(renderer.glyph).__name__
+                                     for renderer in incompatible_renderers])
+            return "%s glyph type(s) found." % glyph_types
